import { OnInit, Component, ViewChild } from "@angular/core";
import { MatSort, MatPaginator, PageEvent, MatSnackBar } from "@angular/material";
import { DatasetCriteriaComponent } from "../../shared/components/criteria/datasets/datasets-criteria.component";
import { Observable } from "rxjs";
import { BreadcrumbItem } from "../../shared/components/breadcrumb/definition/breadcrumb-item";
import { DatasetService } from "../../services/dataset/dataset.service";
import { Router, ActivatedRoute, Params } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataManagementPlanService } from "../../services/data-management-plan/data-management-plan.service";
import { DatasetCriteria } from "../../models/criteria/dataset/DatasetCriteria";
import { DataSource } from "@angular/cdk/table";
import { DatasetListingModel } from "../../models/datasets/DatasetListingModel";
import { DataTableRequest } from "../../models/data-table/DataTableRequest";
import { FacetSearchCriteriaModel } from "../../models/facet-search/FacetSearchCriteriaModel";


@Component({
  selector: 'app-dataset-public-listing-component',
  templateUrl: 'dataset-public-listing.component.html',
  styleUrls: ['./dataset-public-listing.component.scss'],
})
export class DatasetPublicListingComponent implements OnInit {


  @ViewChild(MatPaginator) _paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  @ViewChild(DatasetCriteriaComponent) criteria: DatasetCriteriaComponent;

  dataSource: DatasetDataSource | null;
  displayedColumns: String[] = ['label', 'dmp', 'profile', 'status', 'description', 'created'];
  pageEvent: PageEvent;
  titlePrefix: String;
  dmpId: string;

  constructor(
    private datasetService: DatasetService,
    private router: Router,
    private languageService: TranslateService,
    public snackBar: MatSnackBar,
    public route: ActivatedRoute,
    public dataManagementPlanService: DataManagementPlanService
  ) {

  }


  ngOnInit() {
    this.route.params.subscribe(async (params: Params) => {
      this.dmpId = params['dmpId'];
      this.criteria.setCriteria(this.getDefaultCriteria(this.dmpId));
      this.refresh();
      this.criteria.setRefreshCallback(() => this.refresh());
      if (this.dmpId != null) {
        let dmp = await this.dataManagementPlanService.getSingle(this.dmpId).toPromise()
        if (params['dmpLabel'] != undefined)
          this.titlePrefix = "for " + params['dmpLabel'];
      }
    });
  }

  onCriteriaChange(event){
    //console.log(event)
    this.criteria.setCriteria(event);
    this.refresh();
  }

  refresh() {
    this.dataSource = new DatasetDataSource(this.datasetService, this._paginator, this.sort, this.languageService, this.snackBar, this.criteria, this.dmpId);
  }

  rowClick(rowId: String) {
    this.router.navigate(['/datasets/publicEdit/' + rowId]);
  }

  getDefaultCriteria(dmpId: String): DatasetCriteria {
    const defaultCriteria = new DatasetCriteria();
    if (dmpId != null) {
      defaultCriteria.dmpIds.push(dmpId);
    }
    return defaultCriteria;
  }

  makeItPublic(id: String) {
    this.datasetService.makeDatasetPublic(id).subscribe();
  }

}

export class DatasetDataSource extends DataSource<DatasetListingModel> {

  totalCount = 0;
  isLoadingResults = false;
  constructor(
    private _service: DatasetService,
    private _paginator: MatPaginator,
    private _sort: MatSort,
    private _languageService: TranslateService,
    private _snackBar: MatSnackBar,
    private _criteria: DatasetCriteriaComponent,
    private dmpId: String
  ) {
    super();

  }

  connect(): Observable<DatasetListingModel[]> {
    const displayDataChanges = [
      this._paginator.page
      //this._sort.matSortChange
    ];


    return Observable.merge(...displayDataChanges)
      .startWith(null)
      .switchMap(() => {
        setTimeout(() => {
          this.isLoadingResults = true;
        });
        const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
        let fields: Array<string> = new Array()
        if (this._sort.active) fields = this._sort.direction === "asc" ? ["+" + this._sort.active] : ["-" + this._sort.active];
        const request = new DataTableRequest<FacetSearchCriteriaModel>(startIndex, this._paginator.pageSize, { fields: fields });
        request.criteria = this._criteria.criteria;
        //if (this.dmpId) request.criteria.allVersions = true;
        return this._service.getPublicPaged(request);
      })
      /*.catch((error: any) => {
        this._snackBar.openFromComponent(SnackBarNotificationComponent, {
          data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
          duration: 3000,
          extraClasses: ['snackbar-warning']
        });
        //this._criteria.criteria.onCallbackError(error);
        return Observable.of(null);
      })*/
      .map(result => {
        setTimeout(() => {
          this.isLoadingResults = false;
        });
        return result;
      })
      .map(result => {
        if (!result) { return []; }
        if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
        return result.data;
      });
  }

  disconnect() {
    // No-op
  }
}
